@@ -22,6 +22,7 @@
 #endif
 
 #include <glib.h>
+#include <common/panel-private.h>
 
 #include "clock-time.h"
 #include "clock.h"
@@ -66,6 +67,7 @@ struct _ClockTimeTimeout
   guint       restart : 1;
   ClockTime  *time;
   guint       time_changed_id;
+  ClockSleepMonitor *sleep_monitor;
 };
 
 enum
@@ -332,12 +334,14 @@ clock_time_timeout_sync (gpointer user_data)
 ClockTimeTimeout *
 clock_time_timeout_new (guint       interval,
                         ClockTime  *time,
+                        ClockSleepMonitor *sleep_monitor,
                         GCallback   c_handler,
                         gpointer    gobject)
 {
   ClockTimeTimeout *timeout;
 
   panel_return_val_if_fail (XFCE_IS_CLOCK_TIME (time), NULL);
+  panel_return_val_if_fail (sleep_monitor == NULL || XFCE_IS_CLOCK_SLEEP_MONITOR (sleep_monitor), NULL);
 
   panel_return_val_if_fail (interval > 0, NULL);
 
@@ -353,6 +357,14 @@ clock_time_timeout_new (guint       interval,
 
   g_object_ref (G_OBJECT (timeout->time));
 
+  if (sleep_monitor != NULL)
+    {
+      timeout->sleep_monitor = sleep_monitor;
+      g_signal_connect_swapped (G_OBJECT (sleep_monitor), "woke-up",
+                                G_CALLBACK (clock_time_timeout_restart), timeout);
+      g_object_ref (G_OBJECT (sleep_monitor));
+    }
+
   clock_time_timeout_set_interval (timeout, interval);
 
   return timeout;
@@ -418,6 +430,19 @@ clock_time_timeout_set_interval (ClockTimeTimeout *timeout,
 
 
 
+void
+clock_time_timeout_restart (ClockTimeTimeout *timeout)
+{
+  panel_return_if_fail (timeout != NULL);
+
+  g_signal_emit (G_OBJECT (timeout->time), clock_time_signals[TIME_CHANGED], 0);
+
+  timeout->restart = 1;
+  clock_time_timeout_set_interval (timeout, timeout->interval);
+}
+
+
+
 void
 clock_time_timeout_free (ClockTimeTimeout *timeout)
 {
@@ -430,8 +455,15 @@ clock_time_timeout_free (ClockTimeTimeout *timeout)
 
   g_object_unref (G_OBJECT (timeout->time));
 
+  if (timeout->sleep_monitor != NULL)
+    {
+      g_signal_handlers_disconnect_by_data (timeout->sleep_monitor, timeout);
+      g_object_unref (G_OBJECT (timeout->sleep_monitor));
+    }
+
   if (G_LIKELY (timeout->timeout_id != 0))
     g_source_remove (timeout->timeout_id);
+
   g_slice_free (ClockTimeTimeout, timeout);
 }
 
